#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using System.Collections.Generic;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Reflection;
using System.Linq;
using System.IO;
using System.Text;
using System.Diagnostics;
#endregion

	#region Enums
	public enum CalculationType
	{
		LookbackAll,
		[Browsable(false)]LookbackDay,
		[Browsable(false)]AccVolSlope
	}
	#endregion
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{

    /// <summary>
    /// Inidicator for estimating PriceSquawk Volume in Time Settings
    /// </summary>
    [Description("Inidicator for estimating PriceSquawk Volume in Time Settings")]
    public class PSVolumeSettings : Indicator
    {

        #region Variables
        // Wizard generated variables
            private int noDaysLookback = 30; // Default setting for NoDaysLookback
			private int noSecondsLookback = 30;
			private DateTime lastDataTimeStamp = DateTime.Now;
			private CalculationType mode = CalculationType.LookbackAll;	
			private double minValue = 0.0;
			private	double midValue = 0.0;
			private double maxValue = 0.0;
			private double avgRelativeBarVolume = 0.0;
			private double avgTotalBarVolume = 0.0;
			private double minimumBandPercent = 0.0;
			private double middleBandPercent = 50.0;
			private double maximumBandPercent = 80.0;
			private double accVolume = 0;
			private double prevAccVolume = 0;
			private double tickVolume = 0;
			private double sensitivityFactor = 0.5;
			private double lastGradient = 0;
			private double prevLQV = 0;
			private double prevUQV = 0;
			private double prevMax = 0;
			private double smoothedContLQV = 0;
			private double smoothedContUQV = 0;
			private double smoothedContMax = 0;
			private double smoothedAvgLQV = 0;
			private double smoothedAvgUQV = 0;
			private double smoothedAvgMax = 0;
			private int smoothedAvgCount = 0;
			private double smoothingFactor = 0.9;
			private int Min,Middle,Max=0;
			private bool prevSkipBar = false;
		
		
			private bool useDayOfWeek = false;
			private bool skipBar = false;
			private List<TnSHistory> tnsHistory = new List<TnSHistory>();
			private string calculatePeriod = "09:45 - 11:45";
			private List<CalculatePeriods>calculatePeriods = new List<CalculatePeriods>();
	
			private DateTime startBarTime = DateTime.MaxValue;
		
			private Dictionary<string, BarCalculation> barHistory = 
					new Dictionary<string, BarCalculation>();
			
			private BarCalculation gradientSeries = new BarCalculation(10);
			private BarCalculation gradientDeltaSeries = new BarCalculation(10);
		
			private SolidBrush				textBrush				= new SolidBrush(Color.Gray);
			private Font 					textFont				= null;
			private int 					textFontSize			= 11;
			private Color 					textColor				= Color.Black;
			private	StringFormat			stringFormatNear		= new StringFormat();
		
        // User defined variables (add any user defined variables below)
			private int barCount = 0;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.Red, 5), PlotStyle.Bar, "MinVolumeBar"));
            Add(new Plot(new Pen(Color.Orange, 5), PlotStyle.Bar, "MiddleVolumeBar"));
            Add(new Plot(new Pen(Color.Lime, 5), PlotStyle.Bar, "MaxVolumeBar"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "MinReferenceValue"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "MidReferenceValue"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "MaxReferenceValue"));
			Add(new Plot(new Pen(Color.Navy, 5), PlotStyle.Bar, "PSVolumeBar"));
			Add(new Plot(new Pen(Color.Red,2),PlotStyle.Line, "MinReferenceAvg"));
			Add(new Plot(new Pen(Color.Orange,2), PlotStyle.Line, "MidReferenceAvg"));
			Add(new Plot(new Pen(Color.Green,2), PlotStyle.Line, "MaxReferenceAvg"));
            Overlay				= false;
			CalculateOnBarClose = true;
			DrawOnPricePanel = false;
        }
		
		protected override void OnStartUp()
		{
			try
			{
				if (!useDayOfWeek)
				{
					for (int i = 0; i < 86400; i++)
					{
						barHistory.Add(i.ToString(),new BarCalculation(noDaysLookback));	
					}
				}
				
				var periods = calculatePeriod.Split(',');
				var displayString = String.Empty;
				
				foreach (var item in periods)
				{
					DateTime datetimeStart, datetimeFinish;
					
					var dates = item.ToString().Split('-');

					
					if (DateTime.TryParse(dates[0],out datetimeStart))
					{
						
						displayString += " Start: " + datetimeStart.ToShortTimeString();
						if (dates.Count() > 1)
						{
							if (DateTime.TryParse(dates[1],out datetimeFinish))
							{
								
								displayString += " Finish: " + datetimeFinish.ToShortTimeString();
								calculatePeriods.Add(new CalculatePeriods {
									StartTime = datetimeStart,
									FinishTime = datetimeFinish
								});	
							}
							else
							{
								calculatePeriods.Add(new CalculatePeriods {
									StartTime = datetimeStart,
									FinishTime = DateTime.MaxValue
								});
							}
						}
						else
						{
							calculatePeriods.Add(new CalculatePeriods {
								StartTime = datetimeStart,
								FinishTime = DateTime.MaxValue
							});
						}
					}
				}
				
	
				
			if(ChartControl != null)
			{
					textColor = ChartControl.AxisColor;
			}
				textBrush = new SolidBrush(textColor);			
				textFont = new Font("Arial", textFontSize,FontStyle.Bold);
				stringFormatNear.Alignment = StringAlignment.Near;
			
				DrawTextFixed("Periods", displayString,
			TextPosition.TopRight, textColor, textFont, Color.Red,Color.Empty,0);
			}
			catch(Exception ex)
			{
				Log(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex), NinjaTrader.Cbi.LogLevel.Error);
			}
			
		}
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			try
			{
				#region LookBackAll
				
				if (mode == CalculationType.LookbackAll)
				{
					if (startBarTime == DateTime.MaxValue)
					{
						startBarTime = Time[0].Date.AddDays(noDaysLookback);
					}
					string barIdentity = Convert.ToInt32(Time[0].TimeOfDay.TotalSeconds).ToString();

					accVolume = 0;						

					var maximumVolume = barHistory[barIdentity].UpperQuartileValue +
                    sensitivityFactor * barHistory[barIdentity].IQR;
	
					smoothedContLQV = prevLQV*smoothingFactor + 
						(1 - smoothingFactor)*barHistory[barIdentity].LowerQuartileValue;
					smoothedContUQV = prevUQV*smoothingFactor + 
						(1 - smoothingFactor)*barHistory[barIdentity].UpperQuartileValue;
					smoothedContMax = prevMax*smoothingFactor +
						(1- smoothingFactor)*maximumVolume;
						
					prevLQV = smoothedContLQV;
					prevUQV = smoothedContUQV;
					prevMax = smoothedContMax;
						
					MinReferenceValue.Set(smoothedContLQV);
					MidReferenceValue.Set(smoothedContUQV);
					MaxReferenceValue.Set(smoothedContMax);

					barHistory[barIdentity].AddNewVolumeData(Volume[0], Time[0]);

					// Set bar colors and calculate band values
					
					if (Time[0].Date.CompareTo(startBarTime)>= 0)
					{

						if (Volume[0] < Min)
						{
							PSVolumeBar.Set(Volume[0]);
						}
						else if(Volume[0] >= Min &&
								Volume[0] < Middle)
						{
							MinVolumeBar.Set(Volume[0]);
						}
						else if (Volume[0] >= Middle &&
							Volume[0] < Max)
						{
							MidVolumeBar.Set(Volume[0]);
						}
						else if(Volume[0] >= Max)
						{
							MaxVolumeBar.Set(Volume[0]);	
						}
						RemoveDrawObject("Lookback error");
						
						
						if(calculatePeriods.Count > 0)
						{
							foreach(var period in calculatePeriods)
							{
								var startTime = period.StartTime;
								var endTime = period.FinishTime;
								
								if (startTime.TimeOfDay.CompareTo(endTime.TimeOfDay) > 0)
								{	
									if (Time[0].TimeOfDay.CompareTo(startTime.TimeOfDay) >=0 ||
									Time[0].TimeOfDay.CompareTo(endTime.TimeOfDay) < 0)
									{
										skipBar = false;
										break;
									}
									else
									{
										skipBar = true;	
									}
								}
								else if (Time[0].TimeOfDay.CompareTo(startTime.TimeOfDay) >=0 &&
									Time[0].TimeOfDay.CompareTo(endTime.TimeOfDay) < 0)
								{
									skipBar = false;
									break;
								}
								else
								{
									skipBar = true;	
								}
							}
						}
						else
						{
							skipBar = false;	
						}

						if (!skipBar)
						{
							DrawTextFixed("Status", "Calculating",
								TextPosition.Center, textColor, textFont, Color.Empty,Color.Empty,0);
							
							smoothedAvgLQV = (smoothedAvgLQV*smoothedAvgCount +  smoothedContLQV)/(smoothedAvgCount + 1);
							smoothedAvgUQV = (smoothedAvgUQV*smoothedAvgCount +  smoothedContUQV)/(smoothedAvgCount + 1);
							smoothedAvgMax = (smoothedAvgMax*smoothedAvgCount +  smoothedContMax)/(smoothedAvgCount + 1);
							smoothedAvgCount++;

			
						}
						
						if (skipBar && !prevSkipBar)
						{
							Min = (int)Math.Floor(smoothedAvgLQV);
							Middle =  (int)Math.Floor(smoothedAvgUQV);
							Max = (int)Math.Floor(smoothedAvgMax);
							
							smoothedAvgLQV = 0;
							smoothedAvgUQV = 0;
							smoothedAvgMax = 0;
							smoothedAvgCount = 0;
							
						DrawTextFixed("Status", "Not Calculating",
							TextPosition.Center, textColor, textFont, Color.Empty,Color.Empty,0);
						}
						MinReferenceAvg.Set(Min);
						MidReferenceAvg.Set(Middle);
						MaxReferenceAvg.Set(Max);
						
						DrawTextFixed("Averages", String.Format("Minimum: {0}  Middle: {1}  Maximum: {2}", Min,Middle,Max),
						TextPosition.BottomRight, Color.White, textFont, Color.Lime,Color.Black,100);	
					}
					else
					{
						DrawTextFixed("Lookback error", "Not enough lookback data", TextPosition.Center, textColor, textFont, Color.Empty, Color.Empty,0);		
					}
							
					prevSkipBar = skipBar;
				}
		
				#endregion
			
				barCount++;	
					
			}
				


			
			catch(Exception ex)
			{
				Log(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex), NinjaTrader.Cbi.LogLevel.Error);
			}
        }
		protected override void OnTermination()
		{
			try
			{
				//WriteToFile();
			}
			catch(Exception ex)
			{
				Log(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex), NinjaTrader.Cbi.LogLevel.Error);
			}
			
		}
	
		#region WriteToFile
		public void WriteToFile()
		{
			try
			{
				string path = @"C:\Users\GRAHAM\Desktop\IndicatorTest";
				
				if (! System.IO.Directory.Exists(path))
				{	
					System.IO.Directory.CreateDirectory(path);	
				}

				using (StreamWriter writer = new StreamWriter(path + @"\barHistory.csv",false))
				{
					writer.WriteLine("ID,Average,Q1,Q3,IQM,IQR,Ol,Ou,Median,Count,Values");
					foreach(var item in barHistory.ToList())
					{
						if (item.Value.Count > 0)
						{
							var line = new StringBuilder();
							line.Append(item.Key + ",");
							line.Append(item.Value.Average.ToString() +",");
							line.Append(item.Value.LowerQuartileValue.ToString() +",");
							line.Append(item.Value.UpperQuartileValue.ToString() +",");
							line.Append(item.Value.IQM.ToString() +",");
							line.Append(item.Value.IQR.ToString() +",");
							line.Append(item.Value.Median.ToString() +",");
							line.Append(item.Value.Count.ToString() + ",");
							foreach(var value in item.Value.VolumeValues)
							{
								line.Append(value.ToString() + ",");	
							}
							
							writer.WriteLine(line.ToString());
						}
					}
				}
			}
			catch(Exception ex)
			{
				throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex));
			}
		}
		#endregion
		#region NestedClasses
		
		public class CalculatePeriods{
			public DateTime StartTime {get; set;}
			public DateTime FinishTime {get;set;}
		}
		
		public class TnSHistory{
			
			public DateTime TimeStamp {get; set;}
			public double Volume {get; set;}
		}
		
		public class DataComparer : IComparer<TnSHistory>
        {

            public int Compare(TnSHistory x, TnSHistory y)
            {
                try
                {
                    if (x == null) return -1;
                    if (y == null) return 1;

                    TnSHistory xItem = x as TnSHistory;
                    TnSHistory yItem = y as TnSHistory;

                    var compareResult = xItem.Volume - yItem.Volume;

                    if (compareResult > 0)
                    {
                        return 1;
                    }
                    else if (compareResult < 0)
                    {
                        return -1;
                    }
                    else
                    {
                        return 0;
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex));
                }
                return 0;
            }
        }
		public class BarCalculation{
			
			public int _lookback = 0;
			public List<TnSHistory> VolumeValues = new List<TnSHistory>();
			
			public double Average = 0;
			public double LowerQuartileValue = 0;
			public double UpperQuartileValue = 0;
		 	public double IQM = 0;
			public double IQR = 0;
			public double Median = 0;
			private double lastLQV = 0;
			private double lastUQV = 0;
			private double lastMaxV = 0;
			private double SF = 0.9;
			private double smoothedLQV = 0;
			private double smoothedUQV = 0;
			private double smoothedMaxV = 0;

			private IComparer<TnSHistory> DataComparer = new DataComparer();
			
			public int Count 
			{
				get { return VolumeValues.Count;}
			}
			public BarCalculation(int lookback)
			{
				_lookback = lookback;
			}
			
			public void AddNewVolumeData(double volume, DateTime date)
			{
				try
				{
					var newItem = new TnSHistory
                    {
                        TimeStamp = date,
                        Volume = volume
                    };

                    var binarySearchIndex = VolumeValues.BinarySearch(newItem, DataComparer);

                    if (binarySearchIndex < 0)
                    {
                        VolumeValues.Insert(~binarySearchIndex, newItem);
                    }
                    else
                    {
                        VolumeValues.Insert(binarySearchIndex, newItem);
                    }
					RemoveOldDataItems(date);
					CalculateAverage(volume);
					CalculateDispersion();
					
						
				}
				catch(Exception ex)
				{
					throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex),ex.InnerException);
				}
			}
			
			public void RemoveOldDataItems(DateTime date)
            {
                try
                {   
                    var startTime = date.Subtract(TimeSpan.FromDays(_lookback));
                    VolumeValues.RemoveAll(x => x.TimeStamp.CompareTo(startTime) <= 0);
                }
                catch (Exception ex)
                {
                    throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex));
                }
            }
			
			public void AddNewCalcData(double data)
			{
				try
				{
					var newItem = new TnSHistory
                    {
                        TimeStamp = DateTime.Now,
                        Volume = data
                    };
					
					VolumeValues.Add(newItem);
					
					if (Count > _lookback)
					{
						CalculateDispersion();
					}
				}
				catch(Exception ex)
				{
					throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex),ex.InnerException);
				}
			}
			private void CalculateAverage(double newValue)
			{
				try
				{
					Average = VolumeValues.Average(x => x.Volume);
				}
				catch(Exception ex)
				{
					throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex));
				}
			}
			
			private void CalculateDispersion()
			{
				try
				{
					//var sortedValues = VolumeValues.OrderBy(x => x).ToList();
					var midpoint = (double)Count / 2;
					
					if (Count >= 2)
					{
						if (Count % 2 != 0)
						{
							var med1 = VolumeValues[(int)Math.Floor(midpoint)].Volume;
							var med2 = VolumeValues[(int)Math.Ceiling(midpoint)].Volume;
							Median = (med1 + med2 ) / 2;
						}
						else
						{
							Median = VolumeValues[Count / 2].Volume;
						}
					}
					var lqvIndex = (int)Math.Floor((double)Count / 4);
					if (lqvIndex < Count)
					{
						LowerQuartileValue = VolumeValues[lqvIndex].Volume;
					}
					var uqvIndex = (int)Math.Ceiling((double)Count*3 / 4);
					if (uqvIndex < Count)
					{
						UpperQuartileValue = VolumeValues[uqvIndex].Volume;
					}
					IQR = UpperQuartileValue - LowerQuartileValue;
					//IQM = sortedValues.Where(x => x >= LowerQuartileValue && x <= UpperQuartileValue).Average();
				
				}
				catch(Exception ex)
				{
					throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex));
				}
			}
			
		}
		#endregion
        #region Properties
		[Description("")]
        [GridCategory("Parameters")]
        public CalculationType Mode
        {
            get { return mode; }
            set { mode = value; }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public String CalculatePeriod
        {
            get { return calculatePeriod; }
            set 
			{
				calculatePeriod = value;
			}
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public int NoDaysLookback
        {
            get { return noDaysLookback; }
            set { noDaysLookback = Math.Max(5, value); }
        }
		[Description("")]
        [GridCategory("Parameters")]
        public double SensitivityFactor
        {
            get { return sensitivityFactor; }
            set { sensitivityFactor = Math.Max(0.00, Math.Min(1.00, value)); }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MinVolumeBar
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MidVolumeBar
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MaxVolumeBar
        {
            get { return Values[2]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MinReferenceValue
        {
            get { return Values[3]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MidReferenceValue
        {
            get { return Values[4]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MaxReferenceValue
        {
            get { return Values[5]; }
        }

	
		[Browsable(false)]
		[XmlIgnore]
        public DataSeries PSVolumeBar
        {
			get { return Values[6]; }
        }
		
				[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MinReferenceAvg
        {
            get { return Values[7]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MidReferenceAvg
        {
            get { return Values[8]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MaxReferenceAvg
        {
            get { return Values[9]; }
        }
		

        #endregion

    }
	

}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private PSVolumeSettings[] cachePSVolumeSettings = null;

        private static PSVolumeSettings checkPSVolumeSettings = new PSVolumeSettings();

        /// <summary>
        /// Inidicator for estimating PriceSquawk Volume in Time Settings
        /// </summary>
        /// <returns></returns>
        public PSVolumeSettings PSVolumeSettings(String calculatePeriod, CalculationType mode, int noDaysLookback, double sensitivityFactor)
        {
            return PSVolumeSettings(Input, calculatePeriod, mode, noDaysLookback, sensitivityFactor);
        }

        /// <summary>
        /// Inidicator for estimating PriceSquawk Volume in Time Settings
        /// </summary>
        /// <returns></returns>
        public PSVolumeSettings PSVolumeSettings(Data.IDataSeries input, String calculatePeriod, CalculationType mode, int noDaysLookback, double sensitivityFactor)
        {
            if (cachePSVolumeSettings != null)
                for (int idx = 0; idx < cachePSVolumeSettings.Length; idx++)
                    if (cachePSVolumeSettings[idx].CalculatePeriod == calculatePeriod && cachePSVolumeSettings[idx].Mode == mode && cachePSVolumeSettings[idx].NoDaysLookback == noDaysLookback && Math.Abs(cachePSVolumeSettings[idx].SensitivityFactor - sensitivityFactor) <= double.Epsilon && cachePSVolumeSettings[idx].EqualsInput(input))
                        return cachePSVolumeSettings[idx];

            lock (checkPSVolumeSettings)
            {
                checkPSVolumeSettings.CalculatePeriod = calculatePeriod;
                calculatePeriod = checkPSVolumeSettings.CalculatePeriod;
                checkPSVolumeSettings.Mode = mode;
                mode = checkPSVolumeSettings.Mode;
                checkPSVolumeSettings.NoDaysLookback = noDaysLookback;
                noDaysLookback = checkPSVolumeSettings.NoDaysLookback;
                checkPSVolumeSettings.SensitivityFactor = sensitivityFactor;
                sensitivityFactor = checkPSVolumeSettings.SensitivityFactor;

                if (cachePSVolumeSettings != null)
                    for (int idx = 0; idx < cachePSVolumeSettings.Length; idx++)
                        if (cachePSVolumeSettings[idx].CalculatePeriod == calculatePeriod && cachePSVolumeSettings[idx].Mode == mode && cachePSVolumeSettings[idx].NoDaysLookback == noDaysLookback && Math.Abs(cachePSVolumeSettings[idx].SensitivityFactor - sensitivityFactor) <= double.Epsilon && cachePSVolumeSettings[idx].EqualsInput(input))
                            return cachePSVolumeSettings[idx];

                PSVolumeSettings indicator = new PSVolumeSettings();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.CalculatePeriod = calculatePeriod;
                indicator.Mode = mode;
                indicator.NoDaysLookback = noDaysLookback;
                indicator.SensitivityFactor = sensitivityFactor;
                Indicators.Add(indicator);
                indicator.SetUp();

                PSVolumeSettings[] tmp = new PSVolumeSettings[cachePSVolumeSettings == null ? 1 : cachePSVolumeSettings.Length + 1];
                if (cachePSVolumeSettings != null)
                    cachePSVolumeSettings.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cachePSVolumeSettings = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Inidicator for estimating PriceSquawk Volume in Time Settings
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PSVolumeSettings PSVolumeSettings(String calculatePeriod, CalculationType mode, int noDaysLookback, double sensitivityFactor)
        {
            return _indicator.PSVolumeSettings(Input, calculatePeriod, mode, noDaysLookback, sensitivityFactor);
        }

        /// <summary>
        /// Inidicator for estimating PriceSquawk Volume in Time Settings
        /// </summary>
        /// <returns></returns>
        public Indicator.PSVolumeSettings PSVolumeSettings(Data.IDataSeries input, String calculatePeriod, CalculationType mode, int noDaysLookback, double sensitivityFactor)
        {
            return _indicator.PSVolumeSettings(input, calculatePeriod, mode, noDaysLookback, sensitivityFactor);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Inidicator for estimating PriceSquawk Volume in Time Settings
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PSVolumeSettings PSVolumeSettings(String calculatePeriod, CalculationType mode, int noDaysLookback, double sensitivityFactor)
        {
            return _indicator.PSVolumeSettings(Input, calculatePeriod, mode, noDaysLookback, sensitivityFactor);
        }

        /// <summary>
        /// Inidicator for estimating PriceSquawk Volume in Time Settings
        /// </summary>
        /// <returns></returns>
        public Indicator.PSVolumeSettings PSVolumeSettings(Data.IDataSeries input, String calculatePeriod, CalculationType mode, int noDaysLookback, double sensitivityFactor)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.PSVolumeSettings(input, calculatePeriod, mode, noDaysLookback, sensitivityFactor);
        }
    }
}
#endregion
